import logging

from xmlschema.validators.simple_types import *
from xmlschema.validators.complex_types import *
from xmlschema.validators.groups import *
from xmlschema.validators.facets import *

from .TypeMapping import TypeMapping
from .ChoiceMapping import ChoiceMapping
from .ComplexTypeMapping import ComplexTypeMapping

log = logging.getLogger()


class SequenceMapping(ComplexTypeMapping):
    def map(self, xst: BaseXsdType):
        log.debug(f"Attempting mapping of {xst} to sequence")
        j = super().map(xst)
        if j is None:
            log.debug("Not a complex type, giving up")
            return None
        content = xst.content
        if (content.model != 'sequence'):
            log.debug("Not a sequence, giving up")
            return None
        mapped_type = {
            'type' : 'object',
            'properties' : {},
            'required' : []
        }

        # Not going to try and do all of this automatically for now
        # Only make insert the xsiType parameter
        if (xst.base_type):
            # mapped_type['__DESCENDENT_OF__'] = TypeMapping.get_ref_for(xst.base_type, xst.namespaces[''])
            mapped_type['properties']['xsiType'] = {
                "type" : "string",
                "enum" : xst.name
            }
            mapped_type['required'].append('xsiType')
        # if xst.abstract:
        #     mapped_type['__ABSTRACT__'] = True
        #     pass

        inner_choice = None
        for c in list(content.iter_model()):
            log.debug(f"Processing model item {c}")
            if type(c) is XsdElement:
                if c.effective_max_occurs != 1:
                    mapped_type['properties'][c.local_name] = {
                        "type" : "array",
                        "items" : TypeMapping.get_type_from_elem(c, xst.namespaces[''])
                    }
                    if c.effective_max_occurs:
                         mapped_type['properties'][c.local_name]['maxContains'] = c.effective_max_occurs
                    if c.effective_min_occurs > 0:
                         mapped_type['properties'][c.local_name]['minContains'] = c.effective_min_occurs
                else:
                    mapped_type['properties'][c.local_name] = TypeMapping.get_type_from_elem(c, xst.namespaces[''])
                    if c.effective_min_occurs == 1:
                        mapped_type['required'].append(c.local_name)
            elif type(c) is XsdGroup:
                if inner_choice:
                    raise Exception (f"Second group '{c.local_name}' encountered in {xst}")
                if c.model != "choice":
                    raise Exception (f"Don't know what to do with inner group {c} in {xst} - not a choice")
                inner_choice = ChoiceMapping.process_choice(c, xst.namespaces[''])
            elif type(c) is XsdAnyElement:
                mapped_type = {}
            else:
                raise Exception(f"Unknown element type {c}")
        if (inner_choice):
            return { 
                'allOf' : [
                    mapped_type,
                    {'oneOf' : inner_choice}
                ]
            }
        else:
            return mapped_type
